/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v5.0
 * (C) Copyright IBM Corp. 2004, 2005. All Rights Reserved
 * ===========================================================================
 */
 
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStore;
import java.security.Security;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManagerFactory;


/*
 * ServerJsse and ClientJsse
 *
 * Sample applications to demonstrate a simple client/server interaction using the new JSSE
 * provider. They will communicate using all enabled cipher suites.
 *
 * For simplicity, the ServerJsse and ClientJsse use the same KeyStore file called "testkeys"
 * protected by the password "passphrase".  In a real application, the client and server 
 * should each have their own KeyStore files.
 *
 * You can build these sample program classes by issuing the following command:
 *
 *     javac ServerJsse.java
 *     javac ClientJsse.java
 *
 *
 * Running the client/server application requires that the Server is started first and then the Client
 * application.
 *
 * To start the ServerJsse, execute the following command:
 *
 *     java ServerJsse
 *
 * The ServerJsse uses port 8050. Once ServerJsse has started it waits for the ClientJsse connection.
 *
 * To start the client, issue the following command:
 * 
 *     java ClientJsse %hostname(optional)%
 *
 * where %hostname% is the name of the host running SeverJsse.  The default is localhost.
 *
 */

public class ServerJsse {
    static int N = 50, L = 100, R = 2;

    ServerJsse() {
        super();
    }

    static SSLContext sslContext;

    // Open up the commonly shared KeyStore to obtain the Trusted Certificates.
    // KeyStore is of type "JKS".  Filename is "testkeys" and password is "passphrase".
    private static void initContext() {
        try {
            if (Security.getProvider("com.ibm.jsse2.IBMJSSEProvider2") == null) {
                // IBMJSSE2 Provider is not installed, install it
                Security.addProvider(new com.ibm.jsse2.IBMJSSEProvider2());
            }

            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(new FileInputStream("testkeys"), "passphrase".toCharArray());
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("IbmX509","IBMJSSE2");
            kmf.init(ks, "passphrase".toCharArray());
            TrustManagerFactory tmf = TrustManagerFactory.getInstance("PKIX","IBMJSSE2");
            tmf.init(ks);
            sslContext = SSLContext.getInstance("SSL_TLS","IBMJSSE2");
            sslContext.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
        } catch (Throwable e) {
            System.out.println("Failed to read testkeys file.");
            e.printStackTrace();
            System.exit(1);
        }
    }

    /* Starts the ServerJsse application.
     * @param args no parameters required
     */
    public static void main(String args[]) {
        int i, j, len;

        if (args.length > 0)
            N = Integer.parseInt(args[0]);
        if (args.length > 1)
            L = Integer.parseInt(args[1]);

        byte buffer[] = new byte[L];


        System.out.println("SERVER: started.");

        initContext();

        try {

            // Get and print the list of supported enabled cipher suites
            System.out.println("ServerJsse: SSL server context created.");
            System.out.println("ServerJsse: supported cipher suites");
            String supported[] = sslContext.getSocketFactory().getSupportedCipherSuites();
            for (i=0; i < supported.length; i++)
                System.out.println(supported[i]);

//            System.out.println("ServerJsse: enabled cipher suites");
//            String enabled[] = sslContext.getSocketFactory().getDefaultCipherSuites();
//            for (i=0; i < enabled.length; i++)
//                System.out.println(enabled[i]);

            // Create an SSL session over port 8050
            SSLServerSocketFactory factory = sslContext.getServerSocketFactory();
            SSLServerSocket ssl_server_sock = (SSLServerSocket)factory.createServerSocket(8050);
            ssl_server_sock.setEnabledCipherSuites(supported);

            SSLSocket ssl_sock; 
            InputStream istr;
            OutputStream ostr;
            long t;
            while (buffer[0] != -1) {
                for (int n=0; n<R; n++) {
                    try {
                        ssl_sock = (SSLSocket)(ssl_server_sock.accept());

                        ssl_sock.startHandshake();
                        SSLSession session = ssl_sock.getSession();
                        System.out.println("\nServerJsse: SSL connection established");
                        System.out.println("   cipher suite:       " + session.getCipherSuite());
                    } catch (IOException se) {
                        System.out.println("\nServerJsse: client connection refused\n" + se);
                        break;
                    }

                    if (L > 0) {
                        istr = ssl_sock.getInputStream();
                        ostr = ssl_sock.getOutputStream();

                        t = System.currentTimeMillis(); 
                        for (j=0;j<N;j++) {
                            for (len = 0;;) {
                                try {
                                    if ((i = istr.read(buffer, len, L-len)) == -1) {
                                        System.out.println("ServerJsse: connection dropped by partner.");
                                        ssl_sock.close();
                                        return;
                                    }
                                } catch (InterruptedIOException e) {
                                    System.out.println("waiting");
                                    continue;
                                }
                                if ((len+=i) == L) break;
                            }
                            ostr.write(buffer, 0, L);
                        }
                        System.out.println("Messages = " + N*2 + "; Time = " +
                                           (System.currentTimeMillis() - t));
                    }
                    ssl_sock.close();
                }
            }
            ssl_server_sock.close();
            System.out.println("\nServerJsse: SSL connection closed.");
            Thread.sleep(1000);

            // plain socket benchmark
            if (L > 0) {
                // Socket bench
                ServerSocket server_sock = new ServerSocket(8050);
                Socket sock = server_sock.accept();
                System.out.println("\nServerJsse: Socket connection accepted.");
                istr = sock.getInputStream();
                ostr = sock.getOutputStream();

                t = System.currentTimeMillis();
                for (j=0;j<N;j++) {
                    for (len = 0;;) {
                        if ((i = istr.read(buffer, len, L-len)) == -1) {
                            System.out.println("ServerJsse: connection dropped by partner.");
                            return;
                        }
                        if ((len+=i) == L) break;
                    }
                    ostr.write(buffer, 0, L);
                }
                System.out.println("Messages = " + N*2 + "; Time = " +
                                   (System.currentTimeMillis() - t));
                sock.close();
                server_sock.close();
                System.out.println("\nServerJsse: Socket connection closed.");
                Thread.sleep(1000);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException f) {
            f.printStackTrace();
        }
        System.out.println("ServerJsse: terminated.");
    }
}
