/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v5.0
 * (C) Copyright IBM Corp. 2004, 2005. All Rights Reserved
 * ===========================================================================
 */
 
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.URLConnection;
import java.security.Security;
import java.util.Properties;


/*
 * UseHttps
 *
 * Sample program to demonstrate accessing an SSL webserver using the java protocol handler of
 * the com.ibm.net.ssl.www2.protocol class.  The URL is specified without the https: prefix.
 * The HTML returned from this site is displayed.
 * 
 * You can build this sample program class by issuing the following command:
 *
 *     javac UseHttps.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *     
 *     java UseHttps %URL% %socksServerName(optional)% %socksPortNumber(optional)%
 *
 * where %URL% is the URL of the webserver without the https: prefix. It is a required parameter.
 *       %socksServer% is an optional parameter representing the socks server.
 *       %socksPortNumber% is an optional parameter representing the socks port number.
 * 
 */
public class UseHttps {

    public static void main(String argv[]) {

        String socksServer = "";
        String socksPort = "1080";
        String usage = "Usage: java UseHttps URL-to-be-read socksServerName(optional) socksPortNumber(optional)";

        if ((argv == null) || (argv.length == 0)) {
            System.out.println(usage);
            return;
        }

        String prefix = "https://";
        // Build the complete URL, including the protocol
        String fullURL = prefix.concat(argv[0]);

        if (argv.length > 3) {
            System.out.println(usage);
            return;
        }

        if (argv.length >= 2) {
            socksServer = argv[1];
            if (argv.length == 3) {
                socksPort = argv[2];
            }
        }

        System.out.println("Computed URL is " + fullURL);



        // Make sure the  IBMJSSE2 Provider is installed before the IBMJSSE provider
        try {
            Security.insertProviderAt(new com.ibm.jsse2.IBMJSSEProvider2(),1);
        } catch (Exception e) {
            System.out.println ("Cannot install provider: " + e.getMessage());
            System.exit(1);
        }

        /*
         *  Add com.ibm.net.ssl.www2.protocol as the java protocol handler.
         */
        Properties properties = System.getProperties();

        String handlers = System.getProperty("java.protocol.handler.pkgs");
        if (handlers == null) {
            // nothing specified yet (expected case)
            properties.put("java.protocol.handler.pkgs", "com.ibm.net.ssl.www2.protocol");
        } else {
            // something already there, put ourselves out front
            properties.put("java.protocol.handler.pkgs", "com.ibm.net.ssl.www2.protocol|".concat(handlers));          
        }

        if (!socksServer.equals("")) {
            // Must do the setup to get to the socks host
            // Could check first to see if the user already specified it on the invocation line
            if (System.getProperty("socksProxyHost") == null) {
                properties.put("socksProxyHost", socksServer);
            }
            if (!socksPort.equals("")) {
                if (System.getProperty("socksProxyPort") == null) {
                    properties.put("socksProxyPort", socksPort);
                }
            }
        }

        System.setProperties(properties); // put the updated properties back in System

        // Need to set the socketfactory to the new JSSE2 provider
        // This will affect all users on the JVM.
        Security.setProperty("ssl.SocketFactory.provider","com.ibm.jsse2.SSLSocketFactoryImpl");

        try {

            URL page = new URL(fullURL); // Process the URL far enough to find the right handler
            // get URLConnection object that represents a connection to the remote object referred
            // to by the URL.
            URLConnection urlc = page.openConnection();
            urlc.setUseCaches(false); // Don't look at possibly cached data
            System.out.println("Content-type = " + urlc.getContentType()); // See what's here
            System.out.println("Content-length = " + urlc.getContentLength()); // See how much of it there is
            // Read the HTML from this connection and print it out
            BufferedReader br = new BufferedReader(new InputStreamReader(urlc.getInputStream(), "8859_1"));
            String buffer = "\n";
            while (buffer != null) {
                try {
                    System.out.println(buffer);
                    buffer = br.readLine();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                    break;
                }
            }
        } catch (MalformedURLException mue) {
            System.out.println(fullURL + " is not a URL that can be resolved");
        } catch (IOException ie) {
            ie.printStackTrace();
        }
    }
}
